﻿using System.Text.RegularExpressions;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace BeamNG_map_options.Screens
{
    /// <summary>
    /// Interaction logic for DupeLevelScreen.xaml
    /// </summary>
    public partial class DupeLevelScreen : Page
    {
        private MainWindow parent = null;
        private bool nagMessage = true; //Don't want to constantly display errors, so we disable them temporarily with this

        public DupeLevelScreen(MainWindow thisParent)
        {
            InitializeComponent();
            parent = thisParent;
        }

        private void Page_Loaded(object sender, RoutedEventArgs e)
        {
            parent.EnableNextButton(false);
        }

        /// <summary>
        /// Check inputs from the map name textbox
        /// </summary>
        private void mapName_KeyUp(object sender, KeyEventArgs e)
        {
            if (mapName.Text != "") //If the textbox has content
            {
                if (Regex.IsMatch(mapName.Text, " ")) //Not allowed spaces, so we'll show an error.
                {
                    parent.EnableNextButton(false);
                    if (nagMessage)
                    {
                        MessageBox.Show("Map names can't have spaces in them. Please remove all spaces to continue.", "Invalid map name", MessageBoxButton.OK, MessageBoxImage.Error);
                        nagMessage = false;
                    }
                }
                else
                {
                    parent.EnableNextButton(true);
                    nagMessage = true;
                }
            }
            else
            {
                parent.EnableNextButton(false);
            }
        }

        private void fixMat_Click(object sender, RoutedEventArgs e)
        {
            //copyMat.IsEnabled = fixMat.IsChecked.Value;
        }

        private void fixShape_Click(object sender, RoutedEventArgs e)
        {
            //copyShape.IsEnabled = fixShape.IsChecked.Value;
        }

        private void fixForest_Click(object sender, RoutedEventArgs e)
        {
            //copyForest.IsEnabled = fixForest.IsChecked.Value;
        }

        private void buttonFixAll_Click(object sender, RoutedEventArgs e)
        {
            fixMat.IsChecked = true;
            copyMat.IsChecked = true;
            //copyMat.IsEnabled = true;

            fixShape.IsChecked = true;
            copyShape.IsChecked = true;
            //copyShape.IsEnabled = true;

            fixNoExtension.IsChecked = true;

            fixForest.IsChecked = true;
            copyForest.IsChecked = true;
            //copyForest.IsEnabled = true;

            fixUnknown.IsChecked = true;
            recoverTerrain.IsChecked = true;
        }

        private void buttonCopyNone_Click(object sender, RoutedEventArgs e)
        {
            fixMat.IsChecked = false;
            copyMat.IsChecked = false;
            //copyMat.IsEnabled = true;

            fixShape.IsChecked = false;
            copyShape.IsChecked = false;
            //copyShape.IsEnabled = true;

            fixNoExtension.IsChecked = false;

            fixForest.IsChecked = false;
            copyForest.IsChecked = false;
            //copyForest.IsEnabled = true;

            fixUnknown.IsChecked = false;
            recoverTerrain.IsChecked = false;
        }

        private void buttonFixMat_Click(object sender, RoutedEventArgs e)
        {
            fixMat.IsChecked = true;
            copyMat.IsChecked = true;
            //copyMat.IsEnabled = true;

            fixNoExtension.IsChecked = true;

            fixUnknown.IsChecked = true;
        }

        private void buttonFixModel_Click(object sender, RoutedEventArgs e)
        {
            fixShape.IsChecked = true;
            copyShape.IsChecked = true;
            //copyShape.IsEnabled = true;

            fixNoExtension.IsChecked = true;

            fixUnknown.IsChecked = true;
        }

        private void buttonInvert_Click(object sender, RoutedEventArgs e)
        {
            fixMat.IsChecked = !fixMat.IsChecked;
            copyMat.IsChecked = !copyMat.IsChecked;
            //copyMat.IsEnabled = !copyMat.IsEnabled;

            fixShape.IsChecked = !fixShape.IsChecked;
            copyShape.IsChecked = !copyShape.IsChecked;
            //copyShape.IsEnabled = !copyShape.IsEnabled;

            fixNoExtension.IsChecked = !fixNoExtension.IsChecked;

            fixForest.IsChecked = !fixForest.IsChecked;
            copyForest.IsChecked = !copyForest.IsChecked;
            //copyForest.IsEnabled = !copyForest.IsEnabled;

            fixUnknown.IsChecked = !fixUnknown.IsChecked;
            recoverTerrain.IsChecked = !recoverTerrain.IsChecked;
        }
    }
}
