﻿using System;
using System.IO;
using System.Text.RegularExpressions;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;

namespace BeamNG_map_options.Screens
{
    /// <summary>
    /// Interaction logic for CopyDataScreen.xaml
    /// </summary>
    public partial class CopyDataScreen : Page
    {
        private MainWindow parent = null;

        private string directory = null, dirNew = null, dirLevelUp = null, regexMaterial = "", regexModel = "";
        private string[] allFiles = null;
        private int currentFile = 0, maxFile = 0;
        private bool copyRunning = false;
        private string[] materials = { ".png", ".PNG", ".dds", ".DDS", ".jpg", ".JPG" }, models = { ".dae", ".DAE", ".prefab", ".PREFAB" };

        private Thread copyThread = null;
        private DispatcherTimer threadTick = new DispatcherTimer();

        public CopyDataScreen(MainWindow thisParent)
        {
            InitializeComponent();
            parent = thisParent;

            threadTick.Tick += new EventHandler(threadTick_Tick);
            threadTick.Interval = new TimeSpan(0, 0, 0, 0, 1);

            /* Create an output string for a regex
             * operation later on. Basically go through all
             * textures and produce a result which allows
             * regex searching of the textures.*/
            for (int i = 0; i < materials.Length; i++)
            {
                if (i != 0)
                {
                    regexMaterial += "|";
                }

                regexMaterial += "(" + Regex.Escape(materials[i]) + ")"; //Escape any special characters
            }

            //Now the same for the models.
            for (int i = 0; i < models.Length; i++)
            {
                if (i != 0)
                {
                    regexModel += "|";
                }

                regexModel += "(" + Regex.Escape(models[i]) + ")";
            }
        }

        private void Page_Loaded(object sender, RoutedEventArgs e)
        {
            parent.EnableNextButton(false);
            parent.EnableBackButton(false);

            if (parent != null)
            {
                if (parent.GetData("directory") != null && (string)parent.GetData("directory") != null) //Directory name exists
                {
                    if (parent.GetData("newName") != null && (string)parent.GetData("newName") != null) //New map name exists
                    {
                        directory = (string)parent.GetData("directory");
                        string newDirectoryName = (string)parent.GetData("newName");

                        if (Directory.Exists(directory)) //Does the directory we need to copy from exist?
                        {
                            allFiles = Directory.GetFiles(directory, "*", SearchOption.AllDirectories); //Get every file in all child directories

                            copyProgress.Maximum = allFiles.Length; //Set the maximum progress to the maximum amount of files we need to scan
                            statusText.Text = "Discovered " + allFiles.Length + " files.";

                            parent.AddLineToLog("[Copy] Copying from " + directory + "."); //Add some stuff to the log
                            parent.AddLineToLog("[Copy] Found " + allFiles.Length + " files to be copied.");

                            dirLevelUp = DirectoryUp(directory); //Go up a directory level
                            dirNew = dirLevelUp + newDirectoryName; //Replace the old directory name with the new one

                            parent.AddLineToLog("[Copy] Copying to " + dirNew + ".");

                            if (!Directory.Exists(dirNew)) //Does the new directory exist?
                            {
                                parent.AddLineToLog("[Copy] Folder does not already exist, creating.");

                                Directory.CreateDirectory(dirNew); //Create the directory.
                                copyFiles(); //Copy the files.
                            }
                            else
                            {
                                parent.AddLineToLog("[Copy] Folder already exists, asking for input.");

                                MessageBoxResult result = MessageBox.Show("This directory already exists, would you like to copy anyway? Note, this will overwrite existing files with the same name.", "Directory exists", MessageBoxButton.YesNo, MessageBoxImage.Question);

                                if (result == MessageBoxResult.No) //Does the user want to replace files already in there?
                                {
                                    parent.AddLineToLog("[Copy] User aborted");
                                    parent.GoBack();
                                }
                                else
                                {
                                    parent.AddLineToLog("[Copy] User accepted");
                                    copyFiles(); //Copy the files.
                                }
                            }
                        }
                    }
                }
            }
            else
            {

            }
        }

        private void copyFiles()
        {
            if (!copyRunning) //Is the copy already running? Just incase, we do another check, don't want to mess anything up.
            {
                parent.AddLineToLog("[Copy] Starting");

                currentFile = -1;
                maxFile = allFiles.Length;

                copyRunning = true;

                threadTick.Start(); //Start ticking through the files.
            }
        }

        public void threadTick_Tick(object sender, EventArgs e)
        {
            if (copyThread == null || !copyThread.IsAlive) //If no files are copying currently in another thread, we'll create a new one
            {
                currentFile++;

                if (currentFile < maxFile) //Make sure we're not passed the last file
                {
                    copyProgress.Value = currentFile + 1;
                    statusText.Text = "Copying (" + allFiles[currentFile] + ") to " + dirNew;

                    parent.AddLineToLog("[Copy] Copying " + allFiles[currentFile] + " to " + dirNew);

                    copyThread = new Thread(new ThreadStart(delegate //Create a new thread to handle copying, without disabling the UI
                    {
                        if (File.Exists(allFiles[currentFile])) //Check if the current file exists or not
                        {
                            string fileLoc = allFiles[currentFile].Replace(directory, ""); //Get rid of the directory structure before it
                            string[] newFolders = fileLoc.Split('\\'); //Split the directory up into individual directory name sections
                            string currentFolderString = dirNew; //Set up the new directory path
                            string fileName = newFolders[newFolders.Length - 1]; //File name will be the last value in the array (as it's on the end of the directory structure)
                            string fileExtension = Path.GetExtension(fileLoc); //Get our extension on the file, to check later

                            if ((Regex.IsMatch(fileExtension, regexMaterial) && parent.GetData("fixMatLoc") != null && !(bool)parent.GetData("fixMatLoc"))) //Is a material
                            {
                                parent.AddLineToLog("[Copy] File is a material. User opted out of copying materials. Ignoring.");
                            }
                            else if ((Regex.IsMatch(fileExtension, regexModel) && parent.GetData("fixShapeLoc") != null && !(bool)parent.GetData("fixShapeLoc"))) //Is a model
                            {
                                parent.AddLineToLog("[Copy] File is a model. User opted out of copying models. Ignoring.");
                            }
                            else
                            {
                                for (int i = 0; i < newFolders.Length - 1; i++) //Go through every folder we split up and add it back on
                                {
                                    if (newFolders[i].Length > 0)
                                    {
                                        currentFolderString += "\\" + newFolders[i];

                                        if (!Directory.Exists(currentFolderString)) //Create a new directory if applicable, otherwise we can't put the file anywhere
                                        {
                                            parent.AddLineToLog("[Copy] Directory " + currentFolderString + " does not exist, creating.");

                                            Directory.CreateDirectory(currentFolderString);
                                        }
                                    }
                                }

                                string newFileDir = currentFolderString + "\\" + fileName; //Stick the filename on the end of the new directory.

                                if (File.Exists(currentFolderString + "\\" + fileName))
                                {
                                    parent.AddLineToLog("[Copy] File already existed at location, replacing.");

                                    File.Delete(currentFolderString + "\\" + fileName);
                                }

                                File.Copy(allFiles[currentFile], currentFolderString + "\\" + fileName); //Copy the file over
                            }
                        }
                    }
                    ));

                    copyThread.Start(); //Start the thread to copy. When it ends the loop will catch it again and start a new file.
                }
                else
                {
                    CopyComplete(); //We're done copying.
                }
            }
        }

        private void CopyComplete()
        {
            threadTick.Stop();

            parent.AddLineToLog("[Copy] Completed copy. Directory is now " + dirNew);

            parent.SetData("directory", dirNew); //Old directory switches to the new directory
            parent.Navigate(new RenameScreen(parent));
        }

        /// <summary>
        /// Move up a directory level
        /// </summary>
        /// <param name="currentDir">Current full directory path</param>
        /// <returns>Directory minus the end path</returns>
        private string DirectoryUp(string currentDir)
        {
            string[] splitDir = currentDir.Split('\\'); //Split up the directory into its parts
            string dirLevelUp = "";

            for (int i = 0; i < splitDir.Length - 1; i++) //Loop through everything, apart from the final directory section
            {
                dirLevelUp += splitDir[i] + "\\";
            }

            return dirLevelUp;
        }
    }
}
