﻿using System.IO;
using System.Text.RegularExpressions;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Forms;

namespace BeamNG_map_options
{
    /// <summary>
    /// Interaction logic for DirectoryScreen.xaml
    /// </summary>
    public partial class DirectoryScreen : Page
    {
        MainWindow parent = null;
        FolderBrowserDialog folderBrowser = new FolderBrowserDialog();

        public DirectoryScreen(MainWindow thisParent)
        {
            InitializeComponent();
            parent = thisParent;
        }

        private void Page_Loaded(object sender, RoutedEventArgs e)
        {
            parent.EnableNextButton(false);
            parent.ClearLog();
        }

        private void buttonSelectFolder_Click(object sender, RoutedEventArgs e)
        {
            folderBrowser = new FolderBrowserDialog(); //Allow the user to browse to a directory
            folderBrowser.ShowNewFolderButton = false;
            folderBrowser.Description = "Select the folder containing the .mis file of the map you wish to modify. The BeamNG directory is usually found in Users/<Your username>/AppData/Local/BeamNG.";
            folderBrowser.SelectedPath = Properties.Settings.Default.Directory; //Jump to an existing directory if the user has used it before, saves time.
            DialogResult result = folderBrowser.ShowDialog();

            if (result == DialogResult.OK) //User has OK'd a directory name
            {
                if (Directory.Exists(folderBrowser.SelectedPath)) //Does the directory exist?
                {
                    Properties.Settings.Default.Directory = folderBrowser.SelectedPath; //Save their selected directory for use the next time.
                    Properties.Settings.Default.Save();

                    string[] filesInDir = Directory.GetFiles(folderBrowser.SelectedPath); //Get all the files in the top level directory only
                    bool foundMap = false;
                    string match = ".*\\.mis"; //Regex match for the .mis file

                    foreach (string file in filesInDir) //Go through every found file
                    {
                        if (Regex.IsMatch(file, match)) //Does the file contain the regex match string (.mis file)?
                        {
                            foundMap = true; //If so, we set this to true.
                        }
                    }

                    if (foundMap) //If we found the map, we can continue
                    {
                        parent.EnableNextButton(true);
                    }
                    else
                    {
                        parent.EnableNextButton(false);
                        System.Windows.MessageBox.Show("Sorry, this is not a valid level folder. Select the folder which contains the .mis file.", "Level not found", MessageBoxButton.OK, MessageBoxImage.Error);
                    }
                }
                else
                {
                    parent.EnableNextButton(false);
                    System.Windows.MessageBox.Show("Sorry, this directory doesn't appear to exist", "Directory not found", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
        }

        public string GetDirectoryName()
        {
            return folderBrowser.SelectedPath;
        }
    }
}
