﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Threading;

namespace BeamNG_map_options.Screens
{
    /// <summary>
    /// Interaction logic for RemoveArtScreen.xaml
    /// </summary>
    public partial class RemoveArtScreen : Page
    {
        private MainWindow parent = null;

        private string directory = null, artDirectory = null, regexModel = "", regexMaterial = "", allowedFilesRegex = "";
        private string[] allFiles = null;
        private int currentFile = 0, maxFile = 0, deleted = 0;
        private long totalSize = 0;
        private bool searchRunning = false, removeRunning = false;
        private string[] materials = { ".png", ".PNG", ".dds", ".DDS", ".jpg", ".JPG" }, models = { ".dae", ".DAE", ".prefab", ".PREFAB" };

        private Dictionary<string, int> filesExisting = new Dictionary<string, int>();

        private Thread searchThread = null, deleteThread = null;
        private DispatcherTimer searchThreadTick = new DispatcherTimer(), deleteThreadTick = new DispatcherTimer();

        public RemoveArtScreen(MainWindow thisParent)
        {
            InitializeComponent();
            parent = thisParent;

            searchThreadTick.Tick += new EventHandler(searchThreadTick_Tick);
            searchThreadTick.Interval = new TimeSpan(0, 0, 0, 0, 1);

            deleteThreadTick.Tick += new EventHandler(deleteThreadTick_Tick);
            deleteThreadTick.Interval = new TimeSpan(0, 0, 0, 0, 1);

            //The following sets up a global regex case for discovering all contents of both materials and models
            allowedFilesRegex = "\".*(";

            int bigRegexCount = 0;

            for (int i = 0; i < materials.Length; i++)
            {
                if (i != 0)
                {
                    regexMaterial += "|";
                }

                if (bigRegexCount != 0)
                {
                    allowedFilesRegex += "|";
                }

                regexMaterial += "(" + Regex.Escape(materials[i]) + ")";
                allowedFilesRegex += "(" + Regex.Escape(materials[i]) + ")";

                bigRegexCount++;
            }

            for (int i = 0; i < models.Length; i++)
            {
                if (i != 0)
                {
                    regexModel += "|";
                }

                if (bigRegexCount != 0)
                {
                    allowedFilesRegex += "|";
                }

                regexModel += "(" + Regex.Escape(models[i]) + ")";
                allowedFilesRegex += "(" + Regex.Escape(models[i]) + ")";

                bigRegexCount++;
            }

            allowedFilesRegex += ")\"";

            //Example output would be something like ".*(.png|.dae)" which looks for everything with those extensions within quotes, so an entire string of a file name
        }

        private void Page_Loaded(object sender, RoutedEventArgs e)
        {
            MessageBoxResult result = MessageBox.Show("This feature is only a test at the moment. It may do some unexpected stuff, so you might want to back up your art folder just incase. Would you like to continue?", "Hang on", MessageBoxButton.YesNo, MessageBoxImage.Question);

            if (result == MessageBoxResult.Yes)
            {
                parent.EnableNextButton(false);
                parent.EnableBackButton(false);

                if (parent != null)
                {
                    if (parent.GetData("directory") != null && (string)parent.GetData("directory") != null) //Has the directory been selected?
                    {
                        directory = (string)parent.GetData("directory");
                        artDirectory = directory + "\\art"; //Discover the art directory

                        if (Directory.Exists(directory) && Directory.Exists(artDirectory)) //Check if the art directory exists
                        {
                            allFiles = Directory.GetFiles(directory, "*", SearchOption.AllDirectories); //Scan the art directory

                            findProgress.Maximum = allFiles.Length; //Set the progress maximum to the total number of found files

                            statusText.Text = "Discovered " + allFiles.Length + " files.";

                            parent.AddLineToLog("[Remove] Found " + allFiles.Length + " files to search.");

                            searchFiles();
                        }
                    }
                }
            }
            else
            {
                parent.Navigate(new OptionsScreen(parent));
            }
        }

        private void searchFiles()
        {
            if (!searchRunning)
            {
                currentFile = -1;
                maxFile = allFiles.Length;

                searchRunning = true;

                searchThreadTick.Start();
            }
        }

        public void searchThreadTick_Tick(object sender, EventArgs e)
        {
            if (searchThread == null || !searchThread.IsAlive)
            {
                currentFile++;

                if (currentFile < maxFile)
                {
                    findProgress.Value = currentFile + 1;
                    statusText.Text = "Searching (" + allFiles[currentFile] + ")";

                    searchThread = new Thread(new ThreadStart(delegate //Create a thread to do all the filesystem stuff, as we don't want to stop the UI thread
                    {
                        string file = allFiles[currentFile]; //Get the selected file

                        if (File.Exists(file)) //Check the file exists
                        {
                            string extension = Path.GetExtension(file); //Get the file extension

                            if (extension == ".mis" || extension == ".cs") //We'll need to open the .mis and .cs files to check for references to files
                            {
                                StreamReader reader = File.OpenText(file); //Open the file
                                List<string> entireFile = new List<string>();
                                string currentLine;

                                while ((currentLine = reader.ReadLine()) != null) //Read each line until we hit the end of the file
                                {
                                    Match matchedString = null;

                                    if (Regex.IsMatch(currentLine, allowedFilesRegex)) //Look for our expected files
                                    {
                                        matchedString = Regex.Match(currentLine, allowedFilesRegex); //Extract the string
                                    }
                                    else if (Regex.IsMatch(currentLine, "\"\\.(\\\\|/).*\"")) //Look for .\ or ./ directory beginnings, where it references a file internally
                                    {
                                        matchedString = Regex.Match(currentLine, "\"\\.(\\\\|/).*\""); //Extract the entire string within quotes
                                    }
                                    else if (Regex.IsMatch(currentLine, "\"levels(\\\\|/).*\"")) //Same as above, but for directory listings beginning with levels\ or levels/ which we may need to rename
                                    {
                                        matchedString = Regex.Match(currentLine, "\"levels(\\\\|/).*\""); //Extract the entire string within quotes
                                    }

                                    if (matchedString != null && matchedString.Value != "") //Have we matched anything?
                                    {
                                        string fileLocation = matchedString.Value.Replace("\"", ""); //Replace quotes (") with nothing

                                        string[] explodedFSlash = fileLocation.Split('/'); //Explode the string by forward slashes
                                        string[] explodedBSlash = fileLocation.Split('\\'); //Again with backslashes, just incase
                                        List<string> splitTotal = new List<string>(); //Will contain the total exploded directory locations
                                        string fileName = "";

                                        if (fileLocation.Contains('\\') || fileLocation.Contains('/')) //If it has slashes in it at all
                                        {
                                            if (explodedFSlash.Length > 1)
                                            {
                                                splitTotal.AddRange(explodedFSlash.ToList<string>()); //Put the found forward slashes into the total list
                                            }

                                            if (explodedBSlash.Length > 1)
                                            {
                                                splitTotal.AddRange(explodedBSlash.ToList<string>()); //Put the found backwards slashes into the total list
                                            }
                                        }
                                        else
                                        {
                                            splitTotal.Add(fileLocation); //Otherwise just add the location as is
                                        }

                                        foreach (string item in splitTotal)
                                        {
                                            fileName = Regex.Replace(item, "\\..*", ""); //Replace everything after the . of the file with nothing
                                        }

                                        if (fileName != null && fileName != "")
                                        {
                                            /* Here we put all the found files into a list
                                             * which can then be used to loop through and find
                                             * all the items we don't want to remove from the PC*/
                                            if (filesExisting.ContainsKey(fileName.ToLower())) //If there's a key for this item already
                                            {
                                                filesExisting[fileName.ToLower()]++; //Just add it up, doesn't do anything but it's useful for later if I do anything
                                            }
                                            else
                                            {
                                                parent.AddLineToLog("[Remove] " + fileName + " exists in the files. Adding to do not remove list.");
                                                filesExisting.Add(fileName.ToLower(), 1); //Add the initial item to the do not remove list for later
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }));

                    searchThread.Start(); //Start searching
                }
                else
                {
                    SearchComplete(); //If the search is done
                }
            }
        }

        private void SearchComplete()
        {
            searchThreadTick.Stop(); //Stop doing the search stuff

            if (!removeRunning)
            {
                removeRunning = true;

                allFiles = Directory.GetFiles(artDirectory, "*", SearchOption.AllDirectories);

                removeProgress.Maximum = allFiles.Length;
                statusText.Text = "Discovered " + allFiles.Length + " files.";

                currentFile = -1;
                maxFile = allFiles.Length;

                //Console.WriteLine("Delete " + maxFile + " in " + artDirectory);

                parent.AddLineToLog("[Remove] Starting.");

                deleteThreadTick.Start(); //Start deleting
            }
        }

        public void deleteThreadTick_Tick(object sender, EventArgs e)
        {
            if (deleteThread == null || !deleteThread.IsAlive) //If the thread has finished with the previous task
            {
                currentFile++;

                if (currentFile < maxFile)
                {
                    removeProgress.Value = currentFile + 1;
                    statusText.Text = "Checking (" + allFiles[currentFile] + ")";

                    parent.AddLineToLog("[Remove] Looking at " + allFiles[currentFile] + ".");

                    deleteThread = new Thread(new ThreadStart(delegate //Create a new delete thread, to delete items on a separate thread to the UI
                    {
                        string file = allFiles[currentFile]; //Current file iteration in the list of files we currently have

                        if (File.Exists(file)) //Check it exists
                        {
                            string extension = Path.GetExtension(file); //Get the extension of the file
                            string fileName = Path.GetFileNameWithoutExtension(file); //Then the name of the file, minus the extension

                            if (regexModel.Contains(extension) && parent.GetData("removeModels") != null && (bool)parent.GetData("removeModels")) //Are we allowed to remove models
                            {
                                if (!filesExisting.ContainsKey(fileName.ToLower())) //Does the do not remove list have this file?
                                {
                                    deleted++; //Increase the total number of deleted files
                                    totalSize += new FileInfo(file).Length; //Increase the total size deleted

                                    parent.AddLineToLog("[Remove] Deleting " + file + ".");
                                    File.Delete(file); //Delete the file
                                }
                            }
                            else if (regexMaterial.Contains(extension) && parent.GetData("removeTextures") != null && (bool)parent.GetData("removeTextures")) //Are we allowed to remove textures?
                            {
                                if (!filesExisting.ContainsKey(fileName.ToLower()))
                                {
                                    deleted++;
                                    totalSize += new FileInfo(file).Length;

                                    parent.AddLineToLog("[Remove] Deleting " + file + ".");
                                    File.Delete(file);
                                }
                            }
                            else if (regexMaterial.Contains(extension) || regexModel.Contains(extension))
                            {
                                parent.AddLineToLog("[Remove] " + file + " is not referenced anywhere, however the user has opted out of deleting this file.");
                            }
                            else
                            {
                                parent.AddLineToLog("[Remove] Keeping " + file + ".");
                            }
                        }
                    }));

                    deleteThread.Start(); //Start deleting
                }
                else
                {
                    DeleteComplete(); //Finished deleting
                }
            }
        }

        private void DeleteComplete()
        {
            deleteThreadTick.Stop();

            MessageBox.Show("Deleted " + deleted + " files. This has saved you " + ((totalSize / 1024) / 1024) + "MB.", "Complete", MessageBoxButton.OK, MessageBoxImage.Information);

            parent.Navigate(new DoneCopyDupeScreen(parent));
        }
    }
}
