﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Threading;
using System.Windows.Threading;
using System.IO;
using System.Text.RegularExpressions;

namespace BeamNG_map_options.Screens
{
    /// <summary>
    /// Interaction logic for RenameScreen.xaml
    /// </summary>
    public partial class RenameScreen : Page
    {
        private MainWindow parent = null;

        private string directory = null, renameKeyName = null, directoryName, regexMaterial = "", regexModel = "", allowedFilesRegex = "";
        private string[] allFiles = null;
        private int currentFile = 0, maxFile = 0;
        private bool renameRunning = false;
        private string[] materials = { ".png", ".PNG", ".dds", ".DDS", ".jpg", ".JPG" }, models = { ".dae", ".DAE", ".prefab", ".PREFAB" };

        private Thread copyThread = null;
        private DispatcherTimer threadTick = new DispatcherTimer();

        public RenameScreen(MainWindow thisParent)
        {
            InitializeComponent();
            parent = thisParent;

            threadTick.Tick += new EventHandler(threadTick_Tick);
            threadTick.Interval = new TimeSpan(0, 0, 0, 0, 1);

            /* Create the main regex code
             * to find both the textures and
             * the models from the separate
             * allowed files lists*/
            allowedFilesRegex = "\".*(";

            int bigRegexCount = 0;

            for (int i = 0; i < materials.Length; i++)
            {
                if (i != 0)
                {
                    regexMaterial += "|";
                }

                if (bigRegexCount != 0)
                {
                    allowedFilesRegex += "|";
                }

                regexMaterial += "(" + Regex.Escape(materials[i]) + ")";
                allowedFilesRegex += "(" + Regex.Escape(materials[i]) + ")";

                bigRegexCount++;
            }

            for (int i = 0; i < models.Length; i++)
            {
                if (i != 0)
                {
                    regexModel += "|";
                }

                if (bigRegexCount != 0)
                {
                    allowedFilesRegex += "|";
                }

                regexModel += "(" + Regex.Escape(models[i]) + ")";
                allowedFilesRegex += "(" + Regex.Escape(models[i]) + ")";

                bigRegexCount++;
            }

            allowedFilesRegex += ")\"";

            //Console.WriteLine(regexMaterial + ", " + regexModel);
        }

        private void Page_Loaded(object sender, RoutedEventArgs e)
        {
            parent.EnableNextButton(false);
            parent.EnableBackButton(false);

            if (parent != null)
            {
                if (parent.GetData("directory") != null && (string)parent.GetData("directory") != null)
                {
                    if (parent.GetData("newName") != null && (string)parent.GetData("newName") != null)
                    {
                        directory = (string)parent.GetData("directory");

                        if (Directory.Exists(directory))
                        {
                            string[] directorySplit = directory.Split('\\');
                            directoryName = directorySplit[directorySplit.Length - 1];
                            string targetName = (string)parent.GetData("newName");
                            string directoryUp = DirectoryUp(directory); //Go up a directory level

                            if (directoryName != targetName) //Move the directory (rename) if the names are different
                            {
                                parent.AddLineToLog("[Rename] Renaming directory from " + directoryName + " to " + targetName + ".");

                                Directory.Move(directory, directoryUp + targetName);
                                directory = directoryUp + targetName;
                                directoryName = targetName;
                            }

                            allFiles = Directory.GetFiles(directory, "*", SearchOption.AllDirectories); //Get every file in all of the child folders
                            string[] misFile = Directory.GetFiles(directory, "*.mis", SearchOption.TopDirectoryOnly); //Get only the .mis file. We'll use this to get the current level name

                            renameProgress.Maximum = allFiles.Length;
                            statusText.Text = "Discovered " + allFiles.Length + " files.";

                            parent.AddLineToLog("[Rename] Found " + allFiles.Length + " files to search.");

                            if (misFile != null) //If there is an actual .mis file, otherwise we won't bother continuing
                            {
                                parent.AddLineToLog("[Rename] Searching for the .mis file for the base name.");

                                foreach (string potentialMisFile in misFile)
                                {
                                    if (Path.GetExtension(potentialMisFile) == ".mis")
                                    {
                                        renameKeyName = Path.GetFileNameWithoutExtension(potentialMisFile);

                                        parent.AddLineToLog("[Rename] Found the .mis file. Base name is " + renameKeyName + ".");
                                    }
                                }

                                if (renameKeyName != null)
                                {
                                    renameFiles(); //Rename everything
                                }
                            }
                        }
                    }
                }
            }
        }
        private void renameFiles()
        {
            if (!renameRunning)
            {
                parent.AddLineToLog("[Rename] Starting.");

                currentFile = -1;
                maxFile = allFiles.Length;

                renameRunning = true;

                threadTick.Start();
            }
        }

        public void threadTick_Tick(object sender, EventArgs e)
        {
            if (copyThread == null || !copyThread.IsAlive) //Make sure the previous iteration of the thread isn't still running. If it is we'll just loop around again anyway
            {
                currentFile++;

                if (currentFile < maxFile) //Make sure we're not at the file limit
                {
                    renameProgress.Value = currentFile + 1;
                    statusText.Text = "Renaming (" + allFiles[currentFile] + ")";

                    parent.AddLineToLog("[Rename] Looking at " + allFiles[currentFile] + ".");

                    copyThread = new Thread(new ThreadStart(delegate //We have this thread so it doesn't slow down the UI while it does slow reading and writing
                    {
                        string file = allFiles[currentFile]; //Get the currently selected file

                        if (File.Exists(file)) //Make sure it exists
                        {
                            string extension = Path.GetExtension(file); //Get the extension of the file

                            switch (extension) //Check its extension
                            {
                                case ".mis":
                                    CheckMisFile(file);
                                    break;
                                case ".cs":
                                    CheckCsFile(file);
                                    break;
                            }

                            CheckGeneralFile(file);
                        }
                    }
                    ));

                    copyThread.Start(); //Start the thread
                }
                else
                {
                    RenameComplete(); //When we're done renaming everything
                }
            }
        }

        /// <summary>
        /// Check a .mis file
        /// </summary>
        /// <param name="fileName">The pathname for the file</param>
        private void CheckMisFile(string fileName)
        {
            parent.AddLineToLog("[Rename] File is a .mis file. Renaming internal data.");

            StreamReader reader = File.OpenText(fileName); //Open the file as a text file
            List<string> entireFile = new List<string>(); //Create this for later, we'll put the entire file into memory eventually
            string currentLine;

            while ((currentLine = reader.ReadLine()) != null) //Read in the document line by line
            {
                currentLine = AddDirectoryToUnreferencedFiles(currentLine, fileName);

                if (Regex.IsMatch(currentLine, "(\\.ter)|(\\.TER)")) //Terrain
                {
                    parent.AddLineToLog("[Rename] Changing location of the .ter reference.");
                    parent.AddLineToLog("[Rename] \t" + currentLine);

                    currentLine = ReplaceDirectoryName(currentLine);
                    currentLine = ReplaceFileNameReferences(currentLine);

                    parent.AddLineToLog("[Rename] \t" + currentLine);
                    parent.AddLineToLog("[Rename] Changed .ter reference.");
                    //Console.WriteLine(currentLine);
                }
                else if (Regex.IsMatch(currentLine, regexMaterial)) //Textures
                {
                    if (parent.GetData("fixMatLoc") != null && (bool)parent.GetData("fixMatLoc"))
                    {
                        parent.AddLineToLog("[Rename] Changing location of the material reference.");
                        parent.AddLineToLog("[Rename] \t" + currentLine);

                        currentLine = ReplaceDirectoryName(currentLine);
                        //currentLine = ReplaceFileNameReferences(currentLine);

                        parent.AddLineToLog("[Rename] \t" + currentLine);
                        parent.AddLineToLog("[Rename] Changed material reference.");
                    }
                    else
                    {
                        parent.AddLineToLog("[Rename] Found material, however user opted out of renaming materials. Ignoring.");
                    }

                    //Add moving if required
                }
                else if (Regex.IsMatch(currentLine, regexModel)) //Models
                {
                    if (parent.GetData("fixShapeLoc") != null && (bool)parent.GetData("fixShapeLoc"))
                    {
                        parent.AddLineToLog("[Rename] Changing location of the model reference.");
                        parent.AddLineToLog("[Rename] \t" + currentLine);

                        currentLine = ReplaceDirectoryName(currentLine);
                        //currentLine = ReplaceFileNameReferences(currentLine);

                        parent.AddLineToLog("[Rename] \t" + currentLine);
                        parent.AddLineToLog("[Rename] Changed model reference");
                    }
                    else
                    {
                        parent.AddLineToLog("[Rename] Found model, however user opted out of renaming models. Ignoring.");
                    }

                    //Add moving if required
                }
                else
                {
                    if (Regex.IsMatch(currentLine, Regex.Escape("/" + renameKeyName + "/")))
                    {
                        if (parent.GetData("fixNoExtension") != null && (bool)parent.GetData("fixNoExtension"))
                        {
                            parent.AddLineToLog("[Rename] Found a potential directory reference. Attempting to change, if it needs changing.");
                            parent.AddLineToLog("[Rename] \t" + currentLine);

                            currentLine = ReplaceDirectoryName(currentLine);
                            currentLine = ReplaceFileNameReferences(currentLine);

                            parent.AddLineToLog("[Rename] \t" + currentLine);
                            parent.AddLineToLog("[Rename] Attempted to change the directory reference.");
                        }
                        else
                        {
                            parent.AddLineToLog("[Rename] Found unknown object without extension, however user opted out of renaming these. Ignoring.");
                        }
                    }
                }

                entireFile.Add(currentLine);
                ////Console.WriteLine(currentLine);
            }

            reader.Close();

            StreamWriter writer = new StreamWriter(fileName);

            parent.AddLineToLog("[Rename] Writing data back to the .mis file (" + entireFile.Count + " lines).");

            foreach (string line in entireFile)
            {
                writer.WriteLine(line);
            }

            writer.Close();
        }

        /// <summary>
        /// Check a .cs file
        /// </summary>
        /// <param name="fileName">The pathname for the file</param>
        private void CheckCsFile(string fileName)
        {
            parent.AddLineToLog("[Rename] File is a .cs file. Renaming internal data.");

            StreamReader reader = File.OpenText(fileName);
            List<string> entireFile = new List<string>();
            string currentLine;
            bool isForest = false, changed = true;

            while ((currentLine = reader.ReadLine()) != null)
            {
                if (Regex.IsMatch(currentLine, Regex.Escape("TSForestItemData")))
                {
                    parent.AddLineToLog("[Rename] File contains forest data.");

                    isForest = true;
                }

                currentLine = AddDirectoryToUnreferencedFiles(currentLine, fileName); //Find all files without defined file locations and add some on

                //Check the line actually comes out as a filename
                if (((!isForest && parent.GetData("fixUnknown") != null && (bool)parent.GetData("fixUnknown")) || (isForest && parent.GetData("fixForestLoc") != null && (bool)parent.GetData("fixForestLoc"))) && (Regex.IsMatch(currentLine, Regex.Escape("/" + renameKeyName + "/")) || Regex.IsMatch(currentLine, allowedFilesRegex) || Regex.IsMatch(currentLine, "\"\\.(\\\\|/)")))
                {
                    if (Regex.IsMatch(currentLine, regexMaterial)) //Textures
                    {
                        if (parent.GetData("fixMatLoc") != null && (bool)parent.GetData("fixMatLoc"))
                        {
                            changed = true;

                            parent.AddLineToLog("[Rename] Changing location of the material reference.");
                            parent.AddLineToLog("[Rename] \t" + currentLine);

                            currentLine = ReplaceDirectoryName(currentLine); //Replace the old directory name with the new one
                            //currentLine = ReplaceFileNameReferences(currentLine);

                            parent.AddLineToLog("[Rename] \t" + currentLine);
                            parent.AddLineToLog("[Rename] Changed material reference.");
                        }
                        else
                        {
                            parent.AddLineToLog("[Rename] Found material, however user opted out of renaming materials. Ignoring.");
                        }

                        //Add moving if required
                    }
                    else if (Regex.IsMatch(currentLine, regexModel)) //Models
                    {
                        if (parent.GetData("fixShapeLoc") != null && (bool)parent.GetData("fixShapeLoc"))
                        {
                            changed = true;

                            parent.AddLineToLog("[Rename] Changing location of the model reference.");
                            parent.AddLineToLog("[Rename] \t" + currentLine);

                            currentLine = ReplaceDirectoryName(currentLine);
                            //currentLine = ReplaceFileNameReferences(currentLine);

                            parent.AddLineToLog("[Rename] \t" + currentLine);
                            parent.AddLineToLog("[Rename] Changed model reference");
                        }
                        else
                        {
                            parent.AddLineToLog("[Rename] Found model, however user opted out of renaming models. Ignoring.");
                        }

                        //Add moving if required
                    }
                    else
                    {
                        if (parent.GetData("fixNoExtension") != null && (bool)parent.GetData("fixNoExtension"))
                        {
                            changed = true;

                            parent.AddLineToLog("[Rename] Found a potential directory reference. Attempting to change, if it needs changing.");
                            parent.AddLineToLog("[Rename] \t" + currentLine);
                            //Console.WriteLine((isForest ? "Forest" : "Not forest") + " - CS File match " + currentLine);

                            currentLine = ReplaceDirectoryName(currentLine);
                            //currentLine = ReplaceFileNameReferences(currentLine);

                            parent.AddLineToLog("[Rename] \t" + currentLine);
                            parent.AddLineToLog("[Rename] Attempted to change directory reference.");
                        }
                        else
                        {
                            parent.AddLineToLog("[Rename] Found unknown object without extension, however user opted out of renaming these. Ignoring.");
                        }
                    }
                }
                else if (!isForest && parent.GetData("fixUnknown") != null && !(bool)parent.GetData("fixUnknown"))
                {
                    parent.AddLineToLog("[Rename] Found directory reference, however user opted out of fixing locations in .cs files. Ignoring.");
                }
                else if (isForest && parent.GetData("fixForestLoc") != null && !(bool)parent.GetData("fixForestLoc"))
                {
                    parent.AddLineToLog("[Rename] Found forest, however the user has opted out of fixing forest brush locations. Ignoring.");
                }
                else
                {
                    //parent.AddLineToLog("[Rename][ERROR] Unknown error occured attempting to edit this file. May need looking at.");
                }

                entireFile.Add(currentLine);
            }

            reader.Close();

            if (changed) //Has anything changed? This has been made redundant (always true), however I left it here for now just incase.
            {
                StreamWriter writer = new StreamWriter(fileName); //Write the file back

                parent.AddLineToLog("[Rename] Writing data back to the .cs file (" + entireFile.Count + " lines).");

                foreach (string line in entireFile) //Line by line in the saved file list
                {
                    writer.WriteLine(line);
                }

                writer.Close();
            }
            else
            {
                parent.AddLineToLog("[Rename] Nothing to change in the .cs file. Ignoring.");
            }
        }

        /// <summary>
        /// Check a general file to see if the name of the file needs renaming.
        /// </summary>
        /// <param name="fileName">The pathname for the file</param>
        private void CheckGeneralFile(string fileName)
        {
            if (Regex.IsMatch(Path.GetFileName(fileName), Regex.Escape(renameKeyName)))
            {
                parent.AddLineToLog("[Rename] Found a reference to the old level in the filename. Changing.");
                parent.AddLineToLog("[Rename] \t" + fileName);

                string newFile = DirectoryUp(fileName) + Regex.Replace(Path.GetFileName(fileName), Regex.Escape(renameKeyName), directoryName); //Attempt to replace an old filename with a new one

                //Console.WriteLine(newFile);

                if (File.Exists(fileName))
                {
                    File.Move(fileName, newFile); //Move = Rename
                }

                parent.AddLineToLog("[Rename] \t" + newFile);
                parent.AddLineToLog("[Rename] Changed filename.");
            }
        }

        private void RenameComplete() //Done renaming
        {
            threadTick.Stop();

            parent.AddLineToLog("[Rename] Completed rename.");

            parent.Navigate(new DoneCopyDupeScreen(parent));
        }

        private string DirectoryUp(string currentDir)
        {
            string[] splitDir = currentDir.Split('\\');
            string dirLevelUp = "";

            for (int i = 0; i < splitDir.Length - 1; i++)
            {
                dirLevelUp += splitDir[i] + "\\";
            }

            return dirLevelUp;
        }

        /// <summary>
        /// Replaces a directory name from an input string to that of the current directory
        /// </summary>
        /// <param name="input">Path name</param>
        /// <returns></returns>
        private string ReplaceDirectoryName(string input)
        {
            return Regex.Replace(input, "levels/.*?/", "levels/" + directoryName + "/");
        }

        /// <summary>
        /// Check if there are any references that are just the old directory name which need replacing
        /// with the new directory name
        /// </summary>
        /// <param name="input">Path name</param>
        /// <returns></returns>
        private string ReplaceFileNameReferences(string input)
        {
            return Regex.Replace(input, Regex.Escape("/" + renameKeyName + "."), "/" + directoryName + ".");
        }

        private string AddDirectoryToUnreferencedFiles(string input, string fileDirectory)
        {
            /* Gives some directory locations to file references that refer to the folder they're in.
             * For example, in materials.cs there may be a file that references a new diffuseMap
             * DiffuseMap[0] = "AsphaltRoad_damage_sml_decal_01.dds";
             * This can create issues, so we add on the directory levels to the start, so it becomes
             * DiffuseMap[0] = "levels/<mapname>/art/road/AsphaltRoad_damage_sml_decal_01.dds";*/

            Match matchedString = null;
            int matchType = 0;
            string regexDotSlash = "\"\\.(\\\\|/).*\""; //Match either .\ or ./ at the beginning of a file reference name. We add the actual directory location here

            if (Regex.IsMatch(input, allowedFilesRegex))
            {
                matchedString = Regex.Match(input, allowedFilesRegex);
                matchType = 1;
            }
            else if (Regex.IsMatch(input, regexDotSlash))
            {
                Console.WriteLine(input);
                matchedString = Regex.Match(input, regexDotSlash);
                matchType = 2;
            }

            if (matchedString != null && matchedString.Value != "")
            {
                string fileName = matchedString.Value.Replace("\"", "");

                if (!Regex.IsMatch(input, "levels/"))
                {
                    fileName = fileName.Replace("./", "");

                    string directoryReplace = Regex.Replace(DirectoryUp(fileDirectory), ".*levels(\\\\|/)", "levels/");
                    directoryReplace = Regex.Replace(directoryReplace, Regex.Escape(directoryName), renameKeyName);
                    directoryReplace = directoryReplace.Replace('\\', '/');

                    string fullDirectory = "\"" + directoryReplace + fileName + "\"";
                    string fullString = input;

                    if (matchType == 1)
                    {
                        fullString = Regex.Replace(input, allowedFilesRegex, fullDirectory);
                    }
                    else
                    {
                        fullString = Regex.Replace(input, regexDotSlash, fullDirectory);
                    }

                    /*Console.WriteLine(fullDirectory);
                    Console.WriteLine(fullString);*/

                    return fullString;
                }
            }

            return input;
        }
    }
}
